#################################################################################
# EHS demo -- demonstration script for EHS ensemble hydrograph separation code
#


# Version 1.4  build 2020.07.15
# Author: James Kirchner, ETH Zurich
#
#
#
# Copyright (C) 2020 ETH Zurich and James Kirchner
# Public use of this script is permitted under GNU General Public License 3 (GPL3); for details see <https://www.gnu.org/licenses/>
#
# READ THIS CAREFULLY:
# ETH Zurich and James Kirchner make ABSOLUTELY NO WARRANTIES OF ANY KIND, including NO WARRANTIES, expressed or implied, that this software is
#    free of errors or is suitable for any particular purpose.  Users are solely responsible for determining the suitability and
#    reliability of this software for their own purposes.
#
# ALSO READ THIS:
# These scripts implement the ensemble hydrograph separation approach as presented in J.W. Kirchner, "Quantifying new water fractions
#    and transit time distributions using ensemble hydrograph separation: theory and benchmark tests", Hydrology and Earth System Sciences, 
#    23, 303-349, 2019.  
# These scripts are further described in J.W. Kirchner and J.L.A. Knapp, "Calculation scripts for ensemble hydrograph
#    separation", Hydrology and Earth System Sciences, 2020.  
# Users should cite both of these papers (the first for the method, and the second for the scripts)


rm(list=ls()) # clear environment


##################################
# set path and read in source code

setwd("~/EHS_demo")  # set this path to the folder where you have the source code EHS_v1.4 and the input file specified by input_filename


source("EHS_v1.4.R")   #replace filename as needed for future versions



########################
# supply input filename

input_filename <- "EHS demo daily input1.txt"     #replace filename as needed for other data sets

############################
# here we read the data file 

data <- read.delim(input_filename, header=TRUE, sep="\t", na.strings=c("NA",".","","#N/A"))    #this is the test data file


##################################################################################################
# overview of "daily demo input1.txt"
# $ date            : Factor w/ 1826 levels "01-Apr-91","01-Apr-92",..:   # date label (not used)
# $ year            : num  91 91 91 91 91 ...                             # decimal year (not used)
# $ p               : num  0.01 0 0 0 0 ...                               # daily precipitation (mm/day)
# $ q               : num  1.95 1.85 1.76 1.69 1.63 ...                   # daily streamflow (mm/day)
# $ Cp              : num  NA NA NA NA NA ...                             # tracer concentration in precipitation (per mil)
# $ Cq              : num  -46.3 NA -44.7 -46.4 -43.9 ...                 # tracer concentration in streamflow (per mil)
# $ pTop20            : int  1 0 0 0 0 0 0 0 0 1 ...                      # flag for top 20% of precipitation volumes
# $ qTop20            : int  1 0 1 0 1 0 0 1 1 1 ...                      # flag for top 20% of streamflow rates
# $ Cp.corrupted.1pct : num  -7.53 -7.46 -11.98 -9 -12.12 ...             # tracer concentration in precipitation, with 1% of points replaced by outliers
# $ Cq.corrupted.1pct : num  -8.56 -8.13 -8.83 -8.78 -9.38 ...            # tracer concentration in streamflow, with 1% of points replaced by outliers
# $ Cp.corrupted.2pct : num  -7.53 -7.46 -11.98 -9 -12.12 ...             # tracer concentration in precipitation, with 2% of points replaced by outliers
# $ Cq.corrupted.2pct : num  -8.56 -8.13 -8.83 -8.78 -9.38 ...            # tracer concentration in streamflow, with 2% of points replaced by outliers
# $ Cp.corrupted.5pct : num  -7.53 -7.46 -11.98 -9 -12.12 ...
# $ Cq.corrupted.5pct : num  -8.56 -8.13 -8.83 -8.78 -9.38 ...
# $ Cp.corrupted.10pct: num  -7.53 -7.46 -11.98 -9 -12.12 ...
# $ Cq.corrupted.10pct: num  -27.2 -8.13 -8.83 -8.78 -9.38 ...
# $ Cp.corrupted.20pct: num  -7.53 -7.46 -11.98 -40.7 -12.12 ...
# $ Cq.corrupted.20pct: num  -27.2 -8.13 -8.83 -8.78 -61.15 ...
# $ Cp.corrupted.30pct: num  -7.53 -7.46 -11.98 -40.7 -35.16 ...
# $ Cq.corrupted.30pct: num  -27.2 -8.13 -8.83 -25.78 -61.15 ...
# $ Cp.corrupted.40pct: num  -7.53 -7.46 -11.98 -40.7 -35.16 ...
# $ Cq.corrupted.40pct: num  -27.2 -8.13 -8.83 -25.78 -61.15 ...
# $ Cp.corrupted.50pct: num  -7.53 -7.46 -28.32 -40.7 -35.16 ...
# $ Cq.corrupted.50pct: num  -27.2 -8.13 -8.83 -25.78 -61.15 ...



# Note that the date and year information is not used here.
# Instead, it is assumed that the rows of the input file represent an evenly spaced set of times.

# now assign these four variables to their columns in the data table

p <- data$p      #precipitation water flux
q <- data$q      #streamflow water flux

Pfilter <- data$pTop20   # filter for top 20% of (above-threshold) precipitation values
Qfilter <- data$qTop20   # filter for top 20% of discharge values


# now set these options
p_thresh <- 1  # this is the threshold precipitation rate (in P units) below which P tracer inputs will be ignored

nttd <- 25   # number of time steps for TTDs

# these are the lag times associated with each time step
TTD_lag <- seq(1, nttd) - 0.5  # here we treat Q and Cq as being instantaneously measured
# if instead they are time-averaged, then replace -0.5 with -0.75 
# (see Kirchner, 2019, but note that here the index starts at 1 rather than 0)



# Note that the analyses below assume that the default options are
# vol_wtd = FALSE
# robust = TRUE
# ser_corr = TRUE





##################################################################################
##################################################################################
##################################################################################
# analyses WITHOUT outliers


# set label for output filenames
output_label <- "EHS demo daily input1 no outliers"  #label for output files


Cp <- data$Cp    #precipitation tracer concentration (or del value)
Cq <- data$Cq    #streamflow tracer concentration (or del value)



###############################################################################
# estimate transit time distributions (TTDs)
TTD <- EHS_TTD(Cp, Cq, p, q, p_threshold=p_thresh, m=nttd-1)

# estimate flow-weighted TTDs
wtd_TTD <- EHS_TTD(Cp, Cq, p, q, p_threshold=p_thresh, m=nttd-1, vol_wtd=TRUE)

# estimate TTDs for top 20% of discharges
qTop20TTD <- EHS_TTD(Cp, Cq, p, q, p_threshold=p_thresh, m=nttd-1, Qfilter=Qfilter)

# estimate TTDs for top 20% of precipitation
pTop20TTD <- EHS_TTD(Cp, Cq, p, q, p_threshold=p_thresh, m=nttd-1, Pfilter=Pfilter)

# write file of TTDs
write.table(data.frame(TTD_lag, TTD$TTD, TTD$TTD_se, TTD$fwd_TTD, TTD$fwd_TTD_se,
                       wtd_TTD$TTD, wtd_TTD$TTD_se, wtd_TTD$fwd_TTD, wtd_TTD$fwd_TTD_se, 
                       qTop20TTD$TTD, qTop20TTD$TTD_se, pTop20TTD$fwd_TTD, pTop20TTD$fwd_TTD_se), paste(output_label, "robust TTDs.txt"), row.names=FALSE, sep="\t")




###############################################################################
# estimate transit time distributions (TTDs) WITHOUT ROBUST ESTIMATION
TTD <- EHS_TTD(Cp, Cq, p, q, p_threshold=p_thresh, m=nttd-1, robust = FALSE)

# estimate flow-weighted TTDs WITHOUT ROBUST ESTIMATION
wtd_TTD <- EHS_TTD(Cp, Cq, p, q, p_threshold=p_thresh, m=nttd-1, vol_wtd=TRUE, robust = FALSE)

# estimate TTDs for top 20% of discharges WITHOUT ROBUST ESTIMATION
qTop20TTD <- EHS_TTD(Cp, Cq, p, q, p_threshold=p_thresh, m=nttd-1, Qfilter=Qfilter, robust = FALSE)

# estimate TTDs for top 20% of precipitation WITHOUT ROBUST ESTIMATION
pTop20TTD <- EHS_TTD(Cp, Cq, p, q, p_threshold=p_thresh, m=nttd-1, Pfilter=Pfilter, robust = FALSE)

# write file of non-robust TTDs
write.table(data.frame(TTD_lag, TTD$TTD, TTD$TTD_se, TTD$fwd_TTD, TTD$fwd_TTD_se, 
                       wtd_TTD$TTD, wtd_TTD$TTD_se, wtd_TTD$fwd_TTD, wtd_TTD$fwd_TTD_se,
                       qTop20TTD$TTD, qTop20TTD$TTD_se, pTop20TTD$fwd_TTD, pTop20TTD$fwd_TTD_se), paste(output_label, "non-robust TTDs.txt"), row.names=FALSE, sep="\t")



###############################################################################
# estimate and write out profile of robust Fnew for different ranges of discharge
profile <- EHS_profile(Cp, Cq, p, q, p_threshold=p_thresh, crit=q)
write.table(data.frame(profile), paste(output_label,"robust profile by Q ranges.txt"), row.names=FALSE, sep="\t")

# estimate and write out profile of robust Fnew for different ranges of precipitation
profile <- EHS_profile(Cp, Cq, p, q, p_threshold=p_thresh, crit=p)
write.table(data.frame(profile), paste(output_label, "robust profile by P ranges.txt"), row.names=FALSE, sep="\t")

# estimate and write out profile of non-robust Fnew for different ranges of discharge
profile <- EHS_profile(Cp, Cq, p, q, robust=FALSE, p_threshold=p_thresh, crit=q)
write.table(data.frame(profile), paste(output_label,"non-robust profile by Q ranges.txt"), row.names=FALSE, sep="\t")

# estimate and write out profile of non-robust Fnew for different ranges of precipitation
profile <- EHS_profile(Cp, Cq, p, q, robust=FALSE, p_threshold=p_thresh, crit=p)
write.table(data.frame(profile), paste(output_label, "non-robust profile by P ranges.txt"), row.names=FALSE, sep="\t")


###############################################################################
# estimate (robust, unweighted) new water fractions and write them to the console
EHS_Fnew(Cp, Cq, p, q, p_threshold=p_thresh)

# estimate (non-robust, unweighted) new water fractions and write them to the console
EHS_Fnew(Cp, Cq, p, q, robust=FALSE, p_threshold=p_thresh)

# estimate (robust, weighted) new water fractions and write them to the console
EHS_Fnew(Cp, Cq, p, q, p_threshold=p_thresh, vol_wtd=TRUE)

# estimate (non-robust, weighted) new water fractions and write them to the console
EHS_Fnew(Cp, Cq, p, q, robust=FALSE, p_threshold=p_thresh, vol_wtd=TRUE)








##################################################################################
##################################################################################
##################################################################################
# analysis WITH 5 percent of points as outliers


# set label for output filenames
output_label <- "EHS demo daily input1 5pct outliers"  #label for output files


Cp <- data$Cp.corrupted.5pct    #precipitation tracer concentration (or del value)
Cq <- data$Cq.corrupted.5pct    #streamflow tracer concentration (or del value)


###############################################################################
# estimate transit time distributions (TTDs)
TTD <- EHS_TTD(Cp, Cq, p, q, p_threshold=p_thresh, m=nttd-1)

# estimate flow-weighted TTDs
wtd_TTD <- EHS_TTD(Cp, Cq, p, q, p_threshold=p_thresh, m=nttd-1, vol_wtd=TRUE)

# estimate TTDs for top 20% of discharges
qTop20TTD <- EHS_TTD(Cp, Cq, p, q, p_threshold=p_thresh, m=nttd-1, Qfilter=Qfilter)

# estimate TTDs for top 20% of precipitation
pTop20TTD <- EHS_TTD(Cp, Cq, p, q, p_threshold=p_thresh, m=nttd-1, Pfilter=Pfilter)

# write file of TTDs
write.table(data.frame(TTD_lag, TTD$TTD, TTD$TTD_se, TTD$fwd_TTD, TTD$fwd_TTD_se,
                       wtd_TTD$TTD, wtd_TTD$TTD_se, wtd_TTD$fwd_TTD, wtd_TTD$fwd_TTD_se, 
                       qTop20TTD$TTD, qTop20TTD$TTD_se, pTop20TTD$fwd_TTD, pTop20TTD$fwd_TTD_se), paste(output_label, "robust TTDs.txt"), row.names=FALSE, sep="\t")




###############################################################################
# estimate transit time distributions (TTDs) WITHOUT ROBUST ESTIMATION
TTD <- EHS_TTD(Cp, Cq, p, q, p_threshold=p_thresh, m=nttd-1, robust = FALSE)

# estimate flow-weighted TTDs WITHOUT ROBUST ESTIMATION
wtd_TTD <- EHS_TTD(Cp, Cq, p, q, p_threshold=p_thresh, m=nttd-1, vol_wtd=TRUE, robust = FALSE)

# estimate TTDs for top 20% of discharges WITHOUT ROBUST ESTIMATION
qTop20TTD <- EHS_TTD(Cp, Cq, p, q, p_threshold=p_thresh, m=nttd-1, Qfilter=Qfilter, robust = FALSE)

# estimate TTDs for top 20% of precipitation WITHOUT ROBUST ESTIMATION
pTop20TTD <- EHS_TTD(Cp, Cq, p, q, p_threshold=p_thresh, m=nttd-1, Pfilter=Pfilter, robust = FALSE)

# write file of non-robust TTDs
write.table(data.frame(TTD_lag, TTD$TTD, TTD$TTD_se, TTD$fwd_TTD, TTD$fwd_TTD_se, 
                       wtd_TTD$TTD, wtd_TTD$TTD_se, wtd_TTD$fwd_TTD, wtd_TTD$fwd_TTD_se,
                       qTop20TTD$TTD, qTop20TTD$TTD_se, pTop20TTD$fwd_TTD, pTop20TTD$fwd_TTD_se), paste(output_label, "non-robust TTDs.txt"), row.names=FALSE, sep="\t")




###############################################################################
# estimate and write out profile of robust Fnew for different ranges of discharge
profile <- EHS_profile(Cp, Cq, p, q, p_threshold=p_thresh, crit=q)
write.table(data.frame(profile), paste(output_label,"robust profile by Q ranges.txt"), row.names=FALSE, sep="\t")

# estimate and write out profile of robust Fnew for different ranges of precipitation
profile <- EHS_profile(Cp, Cq, p, q, p_threshold=p_thresh, crit=p)
write.table(data.frame(profile), paste(output_label, "robust profile by P ranges.txt"), row.names=FALSE, sep="\t")

# estimate and write out profile of non-robust Fnew for different ranges of discharge
profile <- EHS_profile(Cp, Cq, p, q, robust=FALSE, p_threshold=p_thresh, crit=q)
write.table(data.frame(profile), paste(output_label,"non-robust profile by Q ranges.txt"), row.names=FALSE, sep="\t")

# estimate and write out profile of non-robust Fnew for different ranges of precipitation
profile <- EHS_profile(Cp, Cq, p, q, robust=FALSE, p_threshold=p_thresh, crit=p)
write.table(data.frame(profile), paste(output_label, "non-robust profile by P ranges.txt"), row.names=FALSE, sep="\t")


###############################################################################
# estimate (robust, unweighted) new water fractions and write them to the console
EHS_Fnew(Cp, Cq, p, q, p_threshold=p_thresh)

# estimate (non-robust, unweighted) new water fractions and write them to the console
EHS_Fnew(Cp, Cq, p, q, robust=FALSE, p_threshold=p_thresh)

# estimate (robust, weighted) new water fractions and write them to the console
EHS_Fnew(Cp, Cq, p, q, p_threshold=p_thresh, vol_wtd=TRUE)

# estimate (non-robust, weighted) new water fractions and write them to the console
EHS_Fnew(Cp, Cq, p, q, robust=FALSE, p_threshold=p_thresh, vol_wtd=TRUE)
















##################################################################################
##################################################################################
##################################################################################
# Now let's look at daily input 2 (from a benchmark model that gives a humped transit time distribution)


########################
# supply input and output filenames

input_filename <- "EHS demo daily input2.txt"     #replace filename as needed for other data sets

output_label <- "EHS demo daily input2 no outliers"  #label for output files


############################
# here we read the data file 

data <- read.delim(input_filename, header=TRUE, sep="\t", na.strings=c("NA",".","","#N/A"))    #this is the test data file


p <- data$p      #precipitation water flux
q <- data$q      #streamflow water flux

Cp <- data$Cp
Cq <- data$Cq

Pfilter <- data$pTop20   # filter for top 20% of (above-threshold) precipitation values
Qfilter <- data$qTop20   # filter for top 20% of discharge values


# now set these options
p_thresh <- 1  # this is the threshold precipitation rate (in P units) below which P tracer inputs will be ignored

nttd <- 25   # number of time steps for TTDs

# these are the lag times associated with each time step
TTD_lag <- seq(1, nttd) - 0.5  # here we treat Q and Cq as being instantaneously measured



###############################################################################
# estimate transit time distributions (TTDs)
TTD <- EHS_TTD(Cp, Cq, p, q, p_threshold=p_thresh, m=nttd-1)

# estimate flow-weighted TTDs
wtd_TTD <- EHS_TTD(Cp, Cq, p, q, p_threshold=p_thresh, m=nttd-1, vol_wtd=TRUE)

# estimate TTDs for top 20% of discharges
qTop20TTD <- EHS_TTD(Cp, Cq, p, q, p_threshold=p_thresh, m=nttd-1, Qfilter=Qfilter)

# estimate TTDs for top 20% of precipitation
pTop20TTD <- EHS_TTD(Cp, Cq, p, q, p_threshold=p_thresh, m=nttd-1, Pfilter=Pfilter)

# write file of TTDs
write.table(data.frame(TTD_lag, TTD$TTD, TTD$TTD_se, TTD$fwd_TTD, TTD$fwd_TTD_se,
                       wtd_TTD$TTD, wtd_TTD$TTD_se, wtd_TTD$fwd_TTD, wtd_TTD$fwd_TTD_se, 
                       qTop20TTD$TTD, qTop20TTD$TTD_se, pTop20TTD$fwd_TTD, pTop20TTD$fwd_TTD_se), paste(output_label, "robust TTDs.txt"), row.names=FALSE, sep="\t")



###############################################################################
# estimate transit time distributions (TTDs) WITHOUT ROBUST ESTIMATION
TTD <- EHS_TTD(Cp, Cq, p, q, p_threshold=p_thresh, m=nttd-1, robust = FALSE)

# estimate flow-weighted TTDs WITHOUT ROBUST ESTIMATION
wtd_TTD <- EHS_TTD(Cp, Cq, p, q, p_threshold=p_thresh, m=nttd-1, vol_wtd=TRUE, robust = FALSE)

# estimate TTDs for top 20% of discharges WITHOUT ROBUST ESTIMATION
qTop20TTD <- EHS_TTD(Cp, Cq, p, q, p_threshold=p_thresh, m=nttd-1, Qfilter=Qfilter, robust = FALSE)

# estimate TTDs for top 20% of precipitation WITHOUT ROBUST ESTIMATION
pTop20TTD <- EHS_TTD(Cp, Cq, p, q, p_threshold=p_thresh, m=nttd-1, Pfilter=Pfilter, robust = FALSE)

# write file of non-robust TTDs
write.table(data.frame(TTD_lag, TTD$TTD, TTD$TTD_se, TTD$fwd_TTD, TTD$fwd_TTD_se, 
                       wtd_TTD$TTD, wtd_TTD$TTD_se, wtd_TTD$fwd_TTD, wtd_TTD$fwd_TTD_se,
                       qTop20TTD$TTD, qTop20TTD$TTD_se, pTop20TTD$fwd_TTD, pTop20TTD$fwd_TTD_se), paste(output_label, "non-robust TTDs.txt"), row.names=FALSE, sep="\t")






###############################################################################
# estimate and write out profile of robust Fnew for different ranges of discharge
profile <- EHS_profile(Cp, Cq, p, q, p_threshold=p_thresh, crit=q)
write.table(data.frame(profile), paste(output_label,"robust profile by Q ranges.txt"), row.names=FALSE, sep="\t")

# estimate and write out profile of robust Fnew for different ranges of precipitation
profile <- EHS_profile(Cp, Cq, p, q, p_threshold=p_thresh, crit=p)
write.table(data.frame(profile), paste(output_label, "robust profile by P ranges.txt"), row.names=FALSE, sep="\t")

# estimate and write out non-robust profile of Fnew for different ranges of discharge
profile <- EHS_profile(Cp, Cq, p, q, robust=FALSE, p_threshold=p_thresh, crit=q)
write.table(data.frame(profile), paste(output_label,"non-robust profile by Q ranges.txt"), row.names=FALSE, sep="\t")

# estimate and write out non-robust profile of Fnew for different ranges of precipitation
profile <- EHS_profile(Cp, Cq, p, q, robust=FALSE, p_threshold=p_thresh, crit=p)
write.table(data.frame(profile), paste(output_label, "non-robust profile by P ranges.txt"), row.names=FALSE, sep="\t")







#############################################################################
# now the same TTDs, but with 5% outliers

output_label <- "EHS demo daily input2 5pct outliers"  #label for output files

Cp <- data$Cp.corrupted.5pct
Cq <- data$Cq.corrupted.5pct



###############################################################################
# estimate transit time distributions (TTDs)
TTD <- EHS_TTD(Cp, Cq, p, q, p_threshold=p_thresh, m=nttd-1)

# estimate flow-weighted TTDs
wtd_TTD <- EHS_TTD(Cp, Cq, p, q, p_threshold=p_thresh, m=nttd-1, vol_wtd=TRUE)

# estimate TTDs for top 20% of discharges
qTop20TTD <- EHS_TTD(Cp, Cq, p, q, p_threshold=p_thresh, m=nttd-1, Qfilter=Qfilter)

# estimate TTDs for top 20% of precipitation
pTop20TTD <- EHS_TTD(Cp, Cq, p, q, p_threshold=p_thresh, m=nttd-1, Pfilter=Pfilter)

# write file of TTDs
write.table(data.frame(TTD_lag, TTD$TTD, TTD$TTD_se, TTD$fwd_TTD, TTD$fwd_TTD_se,
                       wtd_TTD$TTD, wtd_TTD$TTD_se, wtd_TTD$fwd_TTD, wtd_TTD$fwd_TTD_se, 
                       qTop20TTD$TTD, qTop20TTD$TTD_se, pTop20TTD$fwd_TTD, pTop20TTD$fwd_TTD_se), paste(output_label, "robust TTDs.txt"), row.names=FALSE, sep="\t")




###############################################################################
# estimate transit time distributions (TTDs) WITHOUT ROBUST ESTIMATION
TTD <- EHS_TTD(Cp, Cq, p, q, p_threshold=p_thresh, m=nttd-1, robust = FALSE)

# estimate flow-weighted TTDs WITHOUT ROBUST ESTIMATION
wtd_TTD <- EHS_TTD(Cp, Cq, p, q, p_threshold=p_thresh, m=nttd-1, vol_wtd=TRUE, robust = FALSE)

# estimate TTDs for top 20% of discharges WITHOUT ROBUST ESTIMATION
qTop20TTD <- EHS_TTD(Cp, Cq, p, q, p_threshold=p_thresh, m=nttd-1, Qfilter=Qfilter, robust = FALSE)

# estimate TTDs for top 20% of precipitation WITHOUT ROBUST ESTIMATION
pTop20TTD <- EHS_TTD(Cp, Cq, p, q, p_threshold=p_thresh, m=nttd-1, Pfilter=Pfilter, robust = FALSE)

# write file of non-robust TTDs
write.table(data.frame(TTD_lag, TTD$TTD, TTD$TTD_se, TTD$fwd_TTD, TTD$fwd_TTD_se, 
                       wtd_TTD$TTD, wtd_TTD$TTD_se, wtd_TTD$fwd_TTD, wtd_TTD$fwd_TTD_se,
                       qTop20TTD$TTD, qTop20TTD$TTD_se, pTop20TTD$fwd_TTD, pTop20TTD$fwd_TTD_se), paste(output_label, "non-robust TTDs.txt"), row.names=FALSE, sep="\t")









##################################################################################
##################################################################################
##################################################################################
# Now let's look at weekly input 1

########################
# supply input and output filenames

input_filename <- "EHS demo weekly input1.txt"     #replace filename as needed for other data sets

output_label <- "EHS demo weekly input1 no outliers"  #label for output files


############################
# here we read the data file

data <- read.delim(input_filename, header=TRUE, sep="\t", na.strings=c("NA",".","","#N/A"))    #this is the test data file


p <- data$p      #precipitation water flux
q <- data$q      #streamflow water flux

Cp <- data$Cp
Cq <- data$Cq


# now set these options
p_thresh <- 1  # this is the threshold precipitation rate (in P units) below which P tracer inputs will be ignored

nttd <- 10   # number of time steps for TTDs

# these are the lag times associated with each time step
TTD_lag <- seq(1, nttd) - 0.5  # here we treat Q and Cq as being instantaneously measured



###############################################################################
# estimate transit time distributions (TTDs)
TTD <- EHS_TTD(Cp, Cq, p, q, p_threshold=p_thresh, m=nttd-1)

# estimate flow-weighted TTDs
wtd_TTD <- EHS_TTD(Cp, Cq, p, q, p_threshold=p_thresh, m=nttd-1, vol_wtd=TRUE)

# write file of TTDs
write.table(data.frame(TTD_lag, TTD$TTD, TTD$TTD_se, TTD$fwd_TTD, TTD$fwd_TTD_se,
                       wtd_TTD$TTD, wtd_TTD$TTD_se, wtd_TTD$fwd_TTD, wtd_TTD$fwd_TTD_se ), paste(output_label, "robust TTDs.txt"), row.names=FALSE, sep="\t")

# estimate non-robust transit time distributions (TTDs)
TTD <- EHS_TTD(Cp, Cq, p, q, p_threshold=p_thresh, m=nttd-1, robust=FALSE)

# estimate non-robust flow-weighted TTDs
wtd_TTD <- EHS_TTD(Cp, Cq, p, q, p_threshold=p_thresh, m=nttd-1, vol_wtd=TRUE, robust=FALSE)

# write file of TTDs
write.table(data.frame(TTD_lag, TTD$TTD, TTD$TTD_se, TTD$fwd_TTD, TTD$fwd_TTD_se,
                       wtd_TTD$TTD, wtd_TTD$TTD_se, wtd_TTD$fwd_TTD, wtd_TTD$fwd_TTD_se ), paste(output_label, "non-robust TTDs.txt"), row.names=FALSE, sep="\t")


###############################################################################
# estimate and write out profile of Fnew for different ranges of discharge
profile <- EHS_profile(Cp, Cq, p, q, p_threshold=p_thresh, crit=q)
write.table(data.frame(profile), paste(output_label, "robust profile by Q ranges.txt"), row.names=FALSE, sep="\t")

# estimate and write out profile of Fnew for different ranges of precipitation
profile <- EHS_profile(Cp, Cq, p, q, p_threshold=p_thresh, crit=p)
write.table(data.frame(profile), paste(output_label, "robust profile by P ranges.txt"), row.names=FALSE, sep="\t")

# estimate and write out non-robust profile of Fnew for different ranges of discharge
profile <- EHS_profile(Cp, Cq, p, q, p_threshold=p_thresh, crit=q, robust=FALSE)
write.table(data.frame(profile), paste(output_label, "non-robust profile by Q ranges.txt"), row.names=FALSE, sep="\t")

# estimate and write out non-robust profile of Fnew for different ranges of precipitation
profile <- EHS_profile(Cp, Cq, p, q, p_threshold=p_thresh, crit=p, robust=FALSE)
write.table(data.frame(profile), paste(output_label, "non-robust profile by P ranges.txt"), row.names=FALSE, sep="\t")




















##################################################################################
##################################################################################
##################################################################################
# Now let's look at weekly input 2 (from a benchmark model that gives a humped transit time distribution)

########################
# supply input and output filenames

input_filename <- "EHS demo weekly input2.txt"     #replace filename as needed for other data sets

output_label <- "EHS demo weekly input2 no outliers"  #label for output files


############################
# here we read the data file

data <- read.delim(input_filename, header=TRUE, sep="\t", na.strings=c("NA",".","","#N/A"))    #this is the test data file


p <- data$p      #precipitation water flux
q <- data$q      #streamflow water flux

Cp <- data$Cp
Cq <- data$Cq


# now set these options
p_thresh <- 1  # this is the threshold precipitation rate (in P units) below which P tracer inputs will be ignored

nttd <- 10   # number of time steps for TTDs

# these are the lag times associated with each time step
TTD_lag <- seq(1, nttd) - 0.5  # here we treat Q and Cq as being instantaneously measured



###############################################################################
# estimate transit time distributions (TTDs)
TTD <- EHS_TTD(Cp, Cq, p, q, p_threshold=p_thresh, m=nttd-1)

# estimate flow-weighted TTDs
wtd_TTD <- EHS_TTD(Cp, Cq, p, q, p_threshold=p_thresh, m=nttd-1, vol_wtd=TRUE)

# write file of TTDs
write.table(data.frame(TTD_lag, TTD$TTD, TTD$TTD_se, TTD$fwd_TTD, TTD$fwd_TTD_se,
                       wtd_TTD$TTD, wtd_TTD$TTD_se, wtd_TTD$fwd_TTD, wtd_TTD$fwd_TTD_se ), paste(output_label, "robust TTDs.txt"), row.names=FALSE, sep="\t")

# estimate non-robust transit time distributions (TTDs)
TTD <- EHS_TTD(Cp, Cq, p, q, p_threshold=p_thresh, m=nttd-1, robust=FALSE)

# estimate non-robust flow-weighted TTDs
wtd_TTD <- EHS_TTD(Cp, Cq, p, q, p_threshold=p_thresh, m=nttd-1, vol_wtd=TRUE, robust=FALSE)

# write file of TTDs
write.table(data.frame(TTD_lag, TTD$TTD, TTD$TTD_se, TTD$fwd_TTD, TTD$fwd_TTD_se,
                       wtd_TTD$TTD, wtd_TTD$TTD_se, wtd_TTD$fwd_TTD, wtd_TTD$fwd_TTD_se ), paste(output_label, "non-robust TTDs.txt"), row.names=FALSE, sep="\t")


###############################################################################
# estimate and write out profile of Fnew for different ranges of discharge
profile <- EHS_profile(Cp, Cq, p, q, p_threshold=p_thresh, crit=q)
write.table(data.frame(profile), paste(output_label, "robust profile by Q ranges.txt"), row.names=FALSE, sep="\t")

# estimate and write out profile of Fnew for different ranges of precipitation
profile <- EHS_profile(Cp, Cq, p, q, p_threshold=p_thresh, crit=p)
write.table(data.frame(profile), paste(output_label, "robust profile by P ranges.txt"), row.names=FALSE, sep="\t")

# estimate and write out non-robust profile of Fnew for different ranges of discharge
profile <- EHS_profile(Cp, Cq, p, q, p_threshold=p_thresh, crit=q, robust=FALSE)
write.table(data.frame(profile), paste(output_label, "non-robust profile by Q ranges.txt"), row.names=FALSE, sep="\t")

# estimate and write out non-robust profile of Fnew for different ranges of precipitation
profile <- EHS_profile(Cp, Cq, p, q, p_threshold=p_thresh, crit=p, robust=FALSE)
write.table(data.frame(profile), paste(output_label, "non-robust profile by P ranges.txt"), row.names=FALSE, sep="\t")



